# IBM WebSphere Commerce Vault/Consul

## Introduction
This chart deploys WebSphere Commerce Vault and Consul as the remote configuration center, which stores environment data and acts as the certification agent to auto-issue certification
to each WebSphere Commerce component based on unique service name.  Vault and Consul is used as a simple topology for development and testing but is required before you deploy WebSphere
Commerce V9 application.  It needs to be deployed only once and the service can be available to multiple Commerce environment. The post-install hook of Helm Chart can be enabled as
default to initialize PKI backend.

## Chart Details
This Helm Chart is only for testing and development purpose. If pod be restarted, you will lose all data and configuration. Please refer [guide](
https://www.vaultproject.io/guides/operations/vault-ha-consul.html) of Vault to setup cluster to support production environment.

## Prerequisites
1. This Helm Chart attempts to pull Vault/Consul/Python Docker Image from DockerHub. If you use the default settings,
ensure that your environment can connect to internet. Or you can pull those Docker images to a private Docker Image Repository

Attribute Name |   Docker Image | Usage
------------- | -------------| -------------
common.vaultImage | docker pull docker.io/vault:0.9.5 | Vault Docker Image provides Vault service
common.consulImage | docker pull docker.io/consul:1.0.6  | Consul Docker Image provides Consul service
common.supportContainer | docker pull docker.io/python:3.7 |  Python Docker Image is used to run post-install scripts
test.image | docker pull docker.io/centos:latest | Helm Test command uses Centos Docker Image

> **Note**
python:3.7 / vault:0.9.5  / consul:1.0.6 have been tested.  There is no guarantee that other tags for those docker images will work as expected.

2. When deploying vault/consul, the support container ( python docker image ) automatically downloads and installs the required library. If internet access is not available, you can use below sample Docker file to build support container

```
FROM docker.io/python:3.7
RUN pip install python-jenkins jinja2 hvac kubernetes argparse requests pyyaml
```

3. If you use ICP 3.1+, create image policies for docker.io in Manage->Resource Security->Image Policies.
4. If you deploy this Helm Chart on any non-default namespace with "Auto Create PKI" set to true, the role based access control should be created first. Run following command to set RBAC. This RBAC is also the prerequisite for deploy WebSphere Commerce Version 9 with wc-websphere-commerce Helm Chart. If the RBAC exist, you can ignore this step, because it only needs to be created once in the same namespace.

Here is the sample to create RBAC with default service account on target namespace:
```
export namespace=TargetNameSpace

cat << EOF > ./wcs-rbac.yaml
kind: Role
apiVersion: rbac.authorization.k8s.io/v1
metadata:
  name: commerce-deploy-support-${namespace}
rules:
- apiGroups: [""] 
  resources: ["secrets"]
  verbs: ["get", "watch", "list","create","delete","patch","update"]
- apiGroups: [""] 
  resources: ["persistentvolumeclaims"]
  verbs: ["get", "watch", "list","create","delete","patch","update"]
- apiGroups: [""] 
  resources: ["pods","pods/log"]
  verbs: ["get", "watch", "list","create","delete","patch","update"]
- apiGroups: [""] 
  resources: ["configmaps"]
  verbs: ["get", "watch", "list","create","delete","patch","update"]
---
kind: RoleBinding
apiVersion: rbac.authorization.k8s.io/v1
metadata:
  name: commerce-deploy-support-${namespace}
subjects:
- kind: ServiceAccount
  name: default
  namespace: ${namespace}
roleRef:
  kind: Role
  name: commerce-deploy-support-${namespace}
  apiGroup: rbac.authorization.k8s.io
EOF

kubectl create -f ./wcs-rbac.yaml -n ${namespace}
```

Note: Please change "TargetNameSpace" to name of target namespace.

### PodSecurityPolicy Requirements

This chart requires a PodSecurityPolicy to be bound to the target namespace prior to installation.  Choose either a predefined PodSecurityPolicy or have your cluster administrator setup a custom PodSecurityPolicy for you:
* Predefined PodSecurityPolicy name: [`ibm-privileged-psp`](https://ibm.biz/cpkspec-psp)
* Custom PodSecurityPolicy definition:

> Note: This PodSecurityPolicy just need to be create once in non-default namespace. If it already exist, please ignore.

```
apiVersion: extensions/v1beta1
kind: PodSecurityPolicy
metadata:
  name: commerce-psp
spec:
  hostIPC: false
  allowPrivilegeEscalation: true
  readOnlyRootFilesystem: false
  allowedCapabilities:
  - CHOWN
  - DAC_OVERRIDE
  - FOWNER
  - FSETID
  - KILL
  - SETGID
  - SETUID
  - SETPCAP
  - NET_BIND_SERVICE
  - NET_RAW
  - SYS_CHROOT
  - MKNOD
  - AUDIT_WRITE
  - SETFCAP
  - SYS_RESOURCE
  - IPC_OWNER
  - SYS_NICE
  seLinux:
    rule: RunAsAny
  supplementalGroups:
    rule: RunAsAny
  runAsUser:
    rule: RunAsAny
  fsGroup:
    rule: RunAsAny
  volumes:
  - configMap
  - emptyDir
  - persistentVolumeClaim
  - secret
  forbiddenSysctls:
  - '*'
```

* Custom ClusterRole for the custom PodSecurityPolicy:

```
apiVersion: rbac.authorization.k8s.io/v1
kind: ClusterRole
metadata:
  name: commerce-psp-clusterrole
rules:
- apiGroups:
  - extensions
  resourceNames:
  - commerce-psp
  resources:
  - podsecuritypolicies
  verbs:
  - use

```

The cluster admin can either paste the above PSP and ClusterRole definitions into the create resource screen in the UI or run the following two commands:

- `kubectl create -f <PSP yaml file>`
- `kubectl create clusterrole commerce-psp-clusterrole --verb=use --resource=podsecuritypolicy --resource-name=commerce-psp`

In ICP 3.1, you also need to create the RoleBinding:

- `kubectl create rolebinding commerce-psp-rolebinding --clusterrole=commerce-psp-clusterrole --serviceaccount=<namespace>:default --namespace=<namespace>`

## Resources Required
The resource limitation is not defined for Vault-Consul, since it's used as the associate service to support WebSphere Commerce Version 9 deployment and will not handle high traffic volumes.

## Installing the Chart
```
$ helm install --name=vault-consul ./ibm-websphere-commerce-vaultconsul
```
> Note: Since the post-install process can take a while, you might not see the helm release immediately. `You can check Vault-Consul helm release status after 2-5 mins.`

## Verifying the Chart
use the helm test command: helm test release-name --cleanup=true 

## Uninstalling the Chart
```
$ helm delete vault-consul --purge --tls
```

## Configuration

The following tables lists the configurable parameters of the <CHARTNAME> chart and their default values.

| Parameter                  | Description                                     | Default                                                    |
| -----------------------    | ---------------------------------------------   | ---------------------------------------------------------- |
| `vaultConsul.vaultImage`             | vault docker image                   | `docker.io/vault `                                                        |
| `vaultConsul.vaultTag`         | vault image tag                  | `0.9.5`                                                    |
| `vaultConsul.consulImage`         | consul docker image           | `docker.io/consul`    |
| `vaultConsul.consulTag`                | consul image tag                         | `1.0.6`                                                   |
| `vaultConsul.supportContainer`             | support docker image for deploy | `docker.io/python:3.7`                                                |
| `vaultConsul.autoCreatePKI`     | enable or disable auto create PKI.recommend to keep default value, owtherwise user have to create PKI by manually | `true`|
| `vaultConsul.nodeLabel`      | specify value of node label wc-node-select-flag to let pod be assiged to target node| `nil`
| `test.image`          | docker image for helm chart test                        | `docker.io/centos:latest`                                                    |

## Limitations
1. This Helm Chart deploys Vault/Consul for development purpose. It will not persist data. When your Pod is restarted, you will lose all data on Vault and Consul.
2. For production usage, follow the [guide](https://www.vaultproject.io/guides/operations/vault-ha-consul.html) to setup cluster.
3. AMD64 is the only supported architecture.
4. Because the post-install process requires time, so you may not see the helm release immediately. `Please check Vault-Consul helm release status after 2-5 mins later.`

## Documentation
* [Learn more about Vault](https://www.vaultproject.io/)
* [Learm more about PKI in Vault](https://www.vaultproject.io/docs/secrets/pki/index.html)
* [Learn more about Key-Value management in Vault](https://www.vaultproject.io/docs/secrets/kv/index.html)
* [How to setup Vault/Consul HA](https://www.vaultproject.io/guides/operations/vault-ha-consul.html)